import sys
import os
import subprocess
import shutil
import settings
import hashlib
from verdict_bro_manager import verdict_bro_manager
import pefile
import filetype
from os import listdir
from os.path import isfile, join
import time
import traceback
from globals import NetworkSensorGlobals

logger = NetworkSensorGlobals.LOGGER()

def is_archive(file_path):
    kind = filetype.guess(file_path)
    if kind is None:
        logger.info('Cannot guess file type!')
        return -1
    if kind.extension in settings.ARCHIVE_FORMATS and settings.ARCHIVE_FORMATS[kind.extension] == kind.mime:
        logger.info('File extension: %s' % kind.extension)
        logger.info('File MIME type: %s' % kind.mime)
        return 1
    else:
        return 0


def is_executable(path):
    try:
        # Check if the file is really an exe file (octet-stream could be anything)
        logger.info("filetype check: %s" %path)
        pe = pefile.PE(path)
        magic_header = pe.OPTIONAL_HEADER.Magic
        if magic_header == 0x010B or magic_header == 0x020B:
            return 1
        else:
            return -1
    except:
        pass
        # print traceback.format_exc()

def delete_folder(folder_path):
    try:
        if os.path.exists(folder_path):
            shutil.rmtree(folder_path)
        else:
            logger.error("Error: %s folder not found" % folder_path)
    except:
        pass
        logger.error(traceback.format_exc())

# deletes the file in the path, works only if the script is run sudo
def delete_file(file_path):
    try:
        if os.path.exists(file_path):
            os.remove(file_path)
        else:
            logger.info("Error: %s file not found" % file_path)
    except:
        pass
        logger.error(traceback.format_exc())

# extracts the file & returns the folder path containing the contents
# TEST NEEDED
def extract_recursive(folder_path, fname):
    try:
        temp_path = os.path.join(folder_path, "temp")
        cmd  = "7z e %s -o\\%s -y" %(os.path.join(folder_path, fname), os.path.join(folder_path, "temp"))
        logger.info("Extracting archive into %s" %temp_path)
        # os.system(cmd)

        proc = subprocess.Popen(cmd, stdout = subprocess.PIPE, shell = True)
        out, err = proc.communicate()
        if "Error" in out:
            return None

        return temp_path
    except:
        pass
        logger.error(traceback.format_exc())
        return None

# TEST NEEDED
def is_folder(file_path):
    if os.path.isdir(file_path):
        # print "It is a folder."
        return True
    else:
        return False


def analyze(source_folder_path):
    # get all the file names in the directory
    try:
        catched_files = [f for f in listdir(source_folder_path) if isfile(join(source_folder_path, f))]

    except:
        return

    if len(catched_files) == 0:
        return

    for fil in catched_files:
        logger.info("Processing: %s" %fil)
        current_filename = fil
        current_file_path = os.path.join(source_folder_path, current_filename)

        if is_executable(current_file_path) == 1:
            # Calculate sha1 & sha256 of the file
            with open(current_file_path, 'rb') as f:
                contents = f.read()
                f_sha1 = hashlib.sha1(contents).hexdigest()
                # f_sha256 = hashlib.sha256(contents).hexdigest()
                logger.info("File is executable (sha1: %s)" % f_sha1)
            vbm = verdict_bro_manager()
            # check file w/ sha1
            file_verdict_result = vbm.queryFileVerdict(f_sha1)
            result_dict = eval(file_verdict_result)
            return_code = result_dict["return_code"]

            # we can write all the return codes below. Only 0 and 8 are relevant in here.
            if return_code == 0:
                logger.info("Analysis is already done for this file (sha1: %s)" % f_sha1)
                logger.info("Last analysis date: ", result_dict["last_analysis_date"])
                logger.info("Result message: ", result_dict["result_message"])
                logger.info("--------------------------------------------------------------")
            elif return_code == 100:
                logger.info("Requested API Key is invalid")
                logger.info("--------------------------------------------------------------")
            elif return_code == 104:
                logger.info("Daily operation request limit is reached, please try again later")
                logger.info("--------------------------------------------------------------")
            elif return_code == 8:
                logger.info("Requested file not found (Absent). Scanning the file (sha1: %s)" % f_sha1)
                logger.info("--------------------------------------------------------------")
                logger.info(vbm.scanFile(current_filename, current_file_path))
                logger.info("--------------------------------------------------------------")
        # if it is an archieve, extract contents into temp and set source folder path to .../temp

        # if fil is a folder, set source folder path to the file's path
        elif is_folder(current_file_path):
            analyze(current_file_path)
        else:
            extracted_folder_path = extract_recursive(source_folder_path, current_filename)
            if extracted_folder_path != None:
                analyze(extracted_folder_path)
            logger.info("File is not executable. Skipping...")
            logger.info("--------------------------------------------------------------")

        # delete the file after the analysis ends
        delete_file(current_file_path)

    if source_folder_path != settings.BRO_EXTRACT_FILES_FOLDER:
        delete_folder(source_folder_path)

def init_bro():
    os.system("%s deploy" %settings.BRO_CTL_PATH)

def configure_nic():
    try:
        logger.info("Configuring NIC")
        with open(os.path.join(os.path.dirname(os.path.realpath(__file__)), "nic_conf"), 'r') as f:
            adapter_name = f.read().splitlines()[0]
        os.system("ethtool -K %s gro off rx off tx off gso off" % adapter_name)
    except:
        logger.error("NIC Configuration failed")
        logger.error(traceback.format_exc())

def main():
    analyze(settings.BRO_EXTRACT_FILES_FOLDER)

if __name__ == '__main__':
    init_bro()
    configure_nic()
    while True:
        main()
        # Every 10 min
        time.sleep(20.0)
